--
-- Polanin seeder for LS2013 v2.1
-- author: Saty / www.forum.lsczech.cz
-- date: 06/2013
--

Polanin = {};

function Polanin.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Fillable, specializations);
end;


function Polanin:load(xmlFile)
	self.changeFoldAttachers = SpecializationUtil.callSpecializationsFunction("changeFoldAttachers");
	self.changeFoldWheels = SpecializationUtil.callSpecializationsFunction("changeFoldWheels");
	self.changeOpenCovers = SpecializationUtil.callSpecializationsFunction("changeOpenCovers");
	
	self.firstLoad = true;
	self.foldAttachers = false;
	self.foldWheels = false;
	self.openCovers = false;
	self.foldAttachersNodeInRange = false;
	
	self.foldAttachersNode = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, "vehicle.specPolanin.foldingNode#index"));
	self.transportAnimation = getXMLString(xmlFile, "vehicle.specPolanin.transportAttacher#name");
	self.workAnimation = getXMLString(xmlFile, "vehicle.specPolanin.workAttacher#name");
	self.transportWheelAnimation = getXMLString(xmlFile, "vehicle.specPolanin.transportWheel#name");
	self.supportMoveAnimation = getXMLString(xmlFile, "vehicle.specPolanin.support#name");
	self.tankCoverAnimation = getXMLString(xmlFile, "vehicle.specPolanin.tankCover#name");
	
	-- LOAD transport attacher --
	self.transportAttacherNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.attacherJoint#index"));
	self.transportAttacherRootNode = Utils.getNoNil(Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.attacherJoint#rootNode")), self.components[1].node);
	self.transportAttacherLowerDistanceToGround = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.attacherJoint#lowerDistanceToGround"), 0.7);
	self.transportAttacherUpperDistanceToGround = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.attacherJoint#upperDistanceToGround"), 1.0);
	local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile,	 "vehicle.attacherJoint#transLimitScale"));
	self.transportAttacherTransLimitScale = { Utils.getNoNil(x, 1), Utils.getNoNil(y, 1), Utils.getNoNil(z, 1) };
	
	-- LOAD work attacher --
	self.workAttacherNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.workAttacherJoint#index"));
	self.workAttacherRootNode = Utils.getNoNil(Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.workAttacherJoint#rootNode")), self.components[1].node);
	self.workAttacherLowerDistanceToGround = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.workAttacherJoint#lowerDistanceToGround"), 0.7);
	self.workAttacherUpperDistanceToGround = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.workAttacherJoint#upperDistanceToGround"), 1.0);
	local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile,	 "vehicle.workAttacherJoint#transLimitScale"));
	self.workAttacherTransLimitScale = { Utils.getNoNil(x, 1), Utils.getNoNil(y, 1), Utils.getNoNil(z, 1) };
end;

function Polanin:update(dt)
	if self.isServer and self.firstLoad and not self.foldAttachers then
		self:changeFoldAttachers(self.foldAttachers);
		self.firstLoad = false;
	end;
	
	if self.isClient and self.foldAttachersNodeInRange then
		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
			self:changeOpenCovers(not self.openCovers);
		end;
		if self.openCovers then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("POLANIN_COVER_CLOSE"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
		else
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("POLANIN_COVER_OPEN"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
		end;
		if self.attacherVehicle == nil then
			if not self.foldWheels then 
				if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
					self:changeFoldAttachers(not self.foldAttachers);
				end;
				if self.foldAttachers then
					g_currentMission:addHelpButtonText(string.format(g_i18n:getText("POLANIN_FOLD"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA2);
				else
					g_currentMission:addHelpButtonText(string.format(g_i18n:getText("POLANIN_UNFOLD"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA2);
				end;
			else
				g_currentMission:addExtraPrintText(g_i18n:getText("POLANIN_HELP2"));
			end;
		else
			g_currentMission:addExtraPrintText(g_i18n:getText("POLANIN_HELP1")); 
		end;
	end;
	
	if self:getIsActiveForInput() and self.foldAttachers then
		if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA2) then
			self:changeFoldWheels(not self.foldWheels);
		end;
		if self.foldWheels then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("POLANIN_WHEEL_UNFOLD"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA2);
		else
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("POLANIN_WHEEL_FOLD"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA2);
		end;
	end;
end;

function Polanin:updateTick(dt)
	if g_currentMission.player ~= nil then
		self.foldAttachersNodeInRange = false;
		local nearestDistance = 5;
		local vx, vy, vz = getWorldTranslation(g_currentMission.player.rootNode);
		local sx, sy, sz = getWorldTranslation(self.foldAttachersNode); 
		local distance = Utils.vector3Length(sx-vx, sy-vy, sz-vz);	
		if distance < nearestDistance then
			self.foldAttachersNodeInRange = true; 
		end;
	end;
	
	if self.foldAttachers then
		self.workAnimationPlay = self:getIsAnimationPlaying(self.workAnimation);
		if not self.workAnimationPlay then
			setVisibility(self.workAttacherRootNode, true);
		end;
	else
		self.transportAnimationPlay = self:getIsAnimationPlaying(self.transportAnimation);
		if not self.transportAnimationPlay then
			setVisibility(self.transportAttacherRootNode, true);
		end;
	end;
	
	if self.isServer then 
		if (self.foldAttachers and not self.foldWheels) or not self.foldAttachers then
			if self.attacherVehicle ~= nil then
				if self.attacherVehicle.isHired then 
					self.attacherVehicle:stopAITractor();
				end;
			end;
			if self.ridgeMarkerState ~= 0 then
				self:setRidgeMarkerState(0);
			end;
		end;
	end;
end;

function Polanin:onAttach(attacherVehicle)
	self.attacherVehicle = attacherVehicle;
	if self.foldAttachers and self.foldWheels then
		if self.supportMoveAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.supportMoveAnimation, -1, nil, true);
		end;
	end;
end;

function Polanin:onDetach()
	self.attacherVehicle = nil;
	if self.foldAttachers and self.foldWheels then
		if self.supportMoveAnimation ~= nil and self.playAnimation ~= nil  then
			self:playAnimation(self.supportMoveAnimation, 1, nil, true);
		end;
	end;
end;

function Polanin:changeFoldAttachers(foldAttachers, noEventSend)
	setFoldAttachersEvent.sendEvent(self, foldAttachers, noEventSend);
	self.foldAttachers = foldAttachers;
	
	if self.foldAttachers then
		self.attacherJoint.node = self.workAttacherNode;
		self.attacherJoint.rootNode = self.workAttacherRootNode;
		self.attacherJoint.transLimitScale = self.workAttacherTransLimitScale;
		self.attacherJoint.lowerDistanceToGround = self.workAttacherLowerDistanceToGround;
		self.attacherJoint.upperDistanceToGround = self.workAttacherUpperDistanceToGround;
		setVisibility(self.transportAttacherRootNode, false);
		if self.workAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.workAnimation, -1, nil, true);
		end;
		if self.transportAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.transportAnimation, 1, nil, true);
		end;
	else
		self.attacherJoint.node = self.transportAttacherNode;
		self.attacherJoint.rootNode = self.transportAttacherRootNode;
		self.attacherJoint.transLimitScale = self.transportAttacherTransLimitScale;
		self.attacherJoint.lowerDistanceToGround = self.transportAttacherLowerDistanceToGround;
		self.attacherJoint.upperDistanceToGround = self.transportAttacherUpperDistanceToGround;
		setVisibility(self.workAttacherRootNode, false);
		if self.transportAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.transportAnimation, -1, nil, true);
		end;
		if self.workAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.workAnimation, 1, nil, true);
		end;
	end;
end;

function Polanin:changeFoldWheels(foldWheels, noEventSend)
	setFoldWheelsEvent.sendEvent(self, foldWheels, noEventSend);
	self.foldWheels = foldWheels;
	
	if self.foldWheels then
		if self.transportWheelAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.transportWheelAnimation, 1, nil, true);
		end;
	else
		if self.transportWheelAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.transportWheelAnimation, -1, nil, true);
		end;
	end;
end;

function Polanin:changeOpenCovers(openCovers, noEventSend)
	setOpenCoversEvent.sendEvent(self, openCovers, noEventSend);
	self.openCovers = openCovers;
	
	if self.openCovers then
		if self.tankCoverAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.tankCoverAnimation, 1, nil, true);
		end;
	else
		if self.tankCoverAnimation ~= nil and self.playAnimation ~= nil then
			self:playAnimation(self.tankCoverAnimation, -1, nil, true);
		end;
	end;
end;

function Polanin:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'foldAttachers="'..tostring(self.foldAttachers)..'" foldWheels="'..tostring(self.foldWheels)..'" openCovers="'..tostring(self.openCovers)..'"';
	return attributes, nil;
end;

function Polanin:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		self:changeFoldAttachers(Utils.getNoNil(getXMLBool(xmlFile, key.."#foldAttachers"), false));
		self:changeFoldWheels(Utils.getNoNil(getXMLBool(xmlFile, key.."#foldWheels"), false));
		self:changeOpenCovers(Utils.getNoNil(getXMLBool(xmlFile, key.."#openCovers"), false));
	end;
	return BaseMission.VEHICLE_LOAD_OK;
end;

function Polanin:readStream(streamId, connection)
	self:changeFoldAttachers(streamReadBool(streamId), true);
	self:changeFoldWheels(streamReadBool(streamId), true);
	self:changeOpenCovers(streamReadBool(streamId), true);
end;

function Polanin:writeStream(streamId, connection)
	streamWriteBool(streamId, self.foldAttachers);
	streamWriteBool(streamId, self.foldWheels);
	streamWriteBool(streamId, self.openCovers);
end;

function Polanin:delete()
end;

function Polanin:draw()
end;

function Polanin:readUpdateStream(streamId, timestamp, connection)
end;

function Polanin:writeUpdateStream(streamId, connection, dirtyMask)
end;

function Polanin:mouseEvent(posX, posY, isDown, isUp, button)
end;

function Polanin:keyEvent(unicode, sym, modifier, isDown)
end;

function Polanin:onLeave()
end;

function Polanin:onDeactivate()
end;


-- All Events --

setFoldAttachersEvent = {};
setFoldAttachersEvent_mt = Class(setFoldAttachersEvent, Event);

InitEventClass(setFoldAttachersEvent, "setFoldAttachersEvent");

function setFoldAttachersEvent:emptyNew()
	local self = Event:new(setFoldAttachersEvent_mt);
	self.className="setFoldAttachersEvent";
	return self;
end;

function setFoldAttachersEvent:new(vehicle, state)
	local self = setFoldAttachersEvent:emptyNew()
	self.vehicle = vehicle;
	self.state = state;
	return self;
end;

function setFoldAttachersEvent:readStream(streamId, connection)
	local id = streamReadInt32(streamId);
	self.state = streamReadBool(streamId);
	self.vehicle = networkGetObject(id);
	self:run(connection);
end;

function setFoldAttachersEvent:writeStream(streamId, connection)
	streamWriteInt32(streamId, networkGetObjectId(self.vehicle));	
	streamWriteBool(streamId, self.state);
end;

function setFoldAttachersEvent:run(connection)
	self.vehicle:changeFoldAttachers(self.state, true);
	if not connection:getIsServer() then
		g_server:broadcastEvent(setFoldAttachersEvent:new(self.vehicle, self.state), nil, connection, self.object);
	end;
end;

function setFoldAttachersEvent.sendEvent(vehicle, state, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(setFoldAttachersEvent:new(vehicle, state), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(setFoldAttachersEvent:new(vehicle, state));
		end;
	end;
end;


setFoldWheelsEvent = {};
setFoldWheelsEvent_mt = Class(setFoldWheelsEvent, Event);

InitEventClass(setFoldWheelsEvent, "setFoldWheelsEvent");

function setFoldWheelsEvent:emptyNew()
	local self = Event:new(setFoldWheelsEvent_mt);
	self.className="setFoldWheelsEvent";
	return self;
end;

function setFoldWheelsEvent:new(vehicle, state)
	local self = setFoldWheelsEvent:emptyNew()
	self.vehicle = vehicle;
	self.state = state;
	return self;
end;

function setFoldWheelsEvent:readStream(streamId, connection)
	local id = streamReadInt32(streamId);
	self.state = streamReadBool(streamId);
	self.vehicle = networkGetObject(id);
	self:run(connection);
end;

function setFoldWheelsEvent:writeStream(streamId, connection)
	streamWriteInt32(streamId, networkGetObjectId(self.vehicle));	
	streamWriteBool(streamId, self.state);
end;

function setFoldWheelsEvent:run(connection)
	self.vehicle:changeFoldWheels(self.state, true);
	if not connection:getIsServer() then
		g_server:broadcastEvent(setFoldWheelsEvent:new(self.vehicle, self.state), nil, connection, self.object);
	end;
end;

function setFoldWheelsEvent.sendEvent(vehicle, state, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(setFoldWheelsEvent:new(vehicle, state), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(setFoldWheelsEvent:new(vehicle, state));
		end;
	end;
end;


setOpenCoversEvent = {};
setOpenCoversEvent_mt = Class(setOpenCoversEvent, Event);

InitEventClass(setOpenCoversEvent, "setOpenCoversEvent");

function setOpenCoversEvent:emptyNew()
	local self = Event:new(setOpenCoversEvent_mt);
	self.className="setOpenCoversEvent";
	return self;
end;

function setOpenCoversEvent:new(vehicle, state)
	local self = setOpenCoversEvent:emptyNew()
	self.vehicle = vehicle;
	self.state = state;
	return self;
end;

function setOpenCoversEvent:readStream(streamId, connection)
	local id = streamReadInt32(streamId);
	self.state = streamReadBool(streamId);
	self.vehicle = networkGetObject(id);
	self:run(connection);
end;

function setOpenCoversEvent:writeStream(streamId, connection)
	streamWriteInt32(streamId, networkGetObjectId(self.vehicle));	
	streamWriteBool(streamId, self.state);
end;

function setOpenCoversEvent:run(connection)
	self.vehicle:changeOpenCovers(self.state, true);
	if not connection:getIsServer() then
		g_server:broadcastEvent(setOpenCoversEvent:new(self.vehicle, self.state), nil, connection, self.object);
	end;
end;

function setOpenCoversEvent.sendEvent(vehicle, state, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(setOpenCoversEvent:new(vehicle, state), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(setOpenCoversEvent:new(vehicle, state));
		end;
	end;
end;